/**
 * @file		main.c
 * @purpose		Example application of vocoder using Speex CODECs.
 * @version		0.1
 * @date		14. 04. 2011
 * @author		CT-PIM
 *----------------------------------------------------------------------------
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 **********************************************************************/

/**********************************************************************/
/* Include files                                                      */
/**********************************************************************/
#include "lpc17xx_libcfg.h"
#include "lpc17xx_gpio.h"
#include "lpc17xx_timer.h"
#include "lpc17xx_clkpwr.h"

#include "spxcodec.h"

uint8_t app_action;

#define ACTION_NONE 0
#define ACTION_REC  1
#define ACTION_PLAY 2

void EINT3_IRQHandler(void)
{
   if (GPIO_GetIntStatus(0, 23, 1))
   {
      GPIO_ClearInt(0,(1<<23));
      app_action = ACTION_REC;
   }
   else if (GPIO_GetIntStatus(2, 13, 1))
   {
      GPIO_ClearInt(2,(1<<13));
      app_action = ACTION_PLAY;
   }
}

/**
 * @brief Main program body
 */
int c_entry(void)
{
   SPX_CODEC_CFG_Type spx_config;

   spx_config.quality    = 4;
   spx_config.complexity = 1;
   spx_config.vbr        = SPX_VBR_DISABLED;
   spx_config.enh        = SPX_ENHANCER_ENABLED;
   
   spx_Init(&spx_config);
   
   /* Configure audio input and output */
   AudioOutput_Init();
   AudioInput_Init();
   
   GPIO_SetDir(1, (1<<25), 1);
   GPIO_ClearValue(1, (1<<25));
   
   GPIO_IntCmd(0,(1<<23),1);
   GPIO_IntCmd(2,(1<<13),1);
   NVIC_EnableIRQ(EINT3_IRQn);
   
   app_action = ACTION_NONE;
   
   while(1)
   {
      /* wait for a user command */
      while(app_action == ACTION_NONE);
      
      GPIO_ClearValue(1, (1<<25));
      if(app_action == ACTION_REC)
      {
         spx_StartEncoder();
      }
      else if (app_action == ACTION_PLAY)
      {
         spx_StartDecoder();
      }
      app_action = ACTION_NONE;
      GPIO_SetValue(1,(1<<25));
   }
   return 1;
}

/* Support required entry point for other toolchain */
int main (void)
{
   return c_entry();
}

#ifdef  DEBUG
/*******************************************************************************
* @brief	Reports the name of the source file and the source line number
* 		where the CHECK_PARAM error has occurred.
* @param[in]	file Pointer to the source file name
* @param[in]    line assert_param error line source number
* @return		None
*******************************************************************************/
void check_failed(uint8_t *file, uint32_t line)
{
   /* User can add his own implementation to report the file name and line number,
   ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while(1);
}
#endif

